@@ -57,9 +57,9 @@ type Table struct {
 	routeFamily  bgp.RouteFamily
 	destinations map[string]*Destination
 	logger       log.Logger
-	// index of route distinguishers with paths to a specific MAC
-	// this is a map[MAC address]map[RD]struct{}
-	// this holds a map for a set of RD.
+	// index of evpn prefixes with paths to a specific MAC
+	// this is a map[MAC address]map[prefix]struct{}
+	// this holds a map for a set of prefixes.
 	macIndex map[string]map[string]struct{}
 }
 
@@ -147,11 +147,10 @@ func (t *Table) deleteDest(dest *Destination) {
 	if nlri, ok := dest.nlri.(*bgp.EVPNNLRI); ok {
 		if macadv, ok := nlri.RouteTypeData.(*bgp.EVPNMacIPAdvertisementRoute); ok {
 			mac := *(*string)(unsafe.Pointer(&macadv.MacAddress))
-			serializedRD, _ := macadv.RD.Serialize()
-			rd := *(*string)(unsafe.Pointer(&serializedRD))
-			if rds, ok := t.macIndex[mac]; ok {
-				delete(rds, rd)
-				if len(rds) == 0 {
+			key := t.tableKey(nlri)
+			if keys, ok := t.macIndex[mac]; ok {
+				delete(keys, key)
+				if len(keys) == 0 {
 					delete(t.macIndex, mac)
 				}
 			}
@@ -171,7 +170,7 @@ func (t *Table) validatePath(path *Path) {
 			log.Fields{
 				"Topic":      "Table",
 				"Key":        t.routeFamily,
-				"Prefix":     path.GetNlri().String(),
+				"Prefix":     path.GetPrefix(),
 				"ReceivedRf": path.GetRouteFamily().String()})
 	}
 	if attr := path.getPathAttr(bgp.BGP_ATTR_TYPE_AS_PATH); attr != nil {
@@ -398,12 +397,11 @@ func (t *Table) setDestination(dst *Destination) {
 	if nlri, ok := dst.nlri.(*bgp.EVPNNLRI); ok {
 		if macadv, ok := nlri.RouteTypeData.(*bgp.EVPNMacIPAdvertisementRoute); ok {
 			mac := *(*string)(unsafe.Pointer(&macadv.MacAddress))
-			serializedRD, _ := macadv.RD.Serialize()
-			rd := *(*string)(unsafe.Pointer(&serializedRD))
-			if rds, ok := t.macIndex[mac]; ok {
-				rds[rd] = struct{}{}
+			key := t.tableKey(nlri)
+			if keys, ok := t.macIndex[mac]; ok {
+				keys[key] = struct{}{}
 			} else {
-				t.macIndex[mac] = map[string]struct{}{rd: {}}
+				t.macIndex[mac] = map[string]struct{}{key: {}}
 			}
 		}
 	}
@@ -435,17 +433,6 @@ func (t *Table) tableKey(nlri bgp.AddrPrefixInterface) string {
 		copy(b[8:24], T.Prefix.To16())
 		b[24] = T.Length
 		return *(*string)(unsafe.Pointer(&b))
-	// we need fast lookup to routes for a specific mac address for evpn mac mobility
-	case *bgp.EVPNNLRI:
-		switch U := T.RouteTypeData.(type) {
-		case *bgp.EVPNMacIPAdvertisementRoute:
-			b := make([]byte, 15)
-			serializedRD, _ := U.RD.Serialize()
-			copy(b, serializedRD)
-			b[8] = bgp.EVPN_ROUTE_TYPE_MAC_IP_ADVERTISEMENT
-			copy(b[9:15], U.MacAddress)
-			return *(*string)(unsafe.Pointer(&b))
-		}
 	}
 	return nlri.String()
 }
@@ -480,17 +467,16 @@ func (t *Table) GetKnownPathList(id string, as uint32) []*Path {
 	return paths
 }
 
-func (t *Table) GetKnownPathListWithMac(id string, as uint32, mac net.HardwareAddr) []*Path {
+func (t *Table) GetKnownPathListWithMac(id string, as uint32, mac net.HardwareAddr, onlyBest bool) []*Path {
 	var paths []*Path
-	if rds, ok := t.macIndex[*(*string)(unsafe.Pointer(&mac))]; ok {
-		for rd := range rds {
-			b := make([]byte, 15)
-			copy(b, rd)
-			b[8] = bgp.EVPN_ROUTE_TYPE_MAC_IP_ADVERTISEMENT
-			copy(b[9:15], mac)
-			key := *(*string)(unsafe.Pointer(&b))
-			if dst, ok := t.destinations[key]; ok {
-				paths = append(paths, dst.GetKnownPathList(id, as)...)
+	if prefixes, ok := t.macIndex[*(*string)(unsafe.Pointer(&mac))]; ok {
+		for prefix := range prefixes {
+			if dst, ok := t.destinations[prefix]; ok {
+				if onlyBest {
+					paths = append(paths, dst.GetBestPath(id, as))
+				} else {
+					paths = append(paths, dst.GetKnownPathList(id, as)...)
+				}
 			}
 		}
 	}
